require 'geocoder/results/base'
#require 'byebug'

module Geocoder::Result
    class MapKit < Base

        def address
            address_components_of_type(:full_address_line)
        end

        def city
            address_components_of_type(:city)
        end

        def state
            ""
        end

        def state_code
            ""
        end

        def country
            address_components_of_type(:country)
        end

        def country_code
            address_components_of_type(:country_code)
        end

        def postal_code
            address_components_of_type(:postal_code)
        end

        def coordinates
            begin
                return [data[1][0]["center"]["lat"], data[1][0]["center"]["lng"]]
            rescue Exception
                return nil
            end
        end


        ##
        # Get address components of a given type.
        #
        #   :street_number
        #   :city
        #   :postal_code
        #   :country
        #   :country_code
        #   :full_address_line
        #   :lat
        #   :lng
        #
        def address_components_of_type(type)
            address_components[type.to_sym]
        end

        # TODO: Parsing of city names, the postal code and street numbers will work in germany only.
        # Street without number:    53.51209, 8.11461
        # Street with number:       53.51413, 8.13440
        # In See:                   48.539185, 9.146325000000001
        # On a Field:               53.506266065100384, 7.90868201732412
        # On a Road (country):      53.50423267292455, 7.908982676402481
        def address_components
            begin
                # Return empty hash if something is not as expected.
                # Don't know whether we have to see geocodeAccuracy == ADDRESS_PARCEL but I'm very cautious here..
                if data[1].blank? || data[1][0].blank? || data[1][0]["formattedAddressLines"].blank?
                    return {}
                end

                formatted_address = data[1][0]["formattedAddressLines"]

                address_hash = {
                    :city               => formatted_address[-2].gsub(formatted_address[-2].split(" ").first, "").strip,
                    :postal_code        => formatted_address[-2].split(" ").first,
                    :full_address_line  => formatted_address.join(","),
                    :country            => data[1][0]["country"],
                    :country_code       => data[1][0]["countryCode"],
                    :lat                => data[1][0]["center"]["lat"],
                    :lng                => data[1][0]["center"]["lng"]
                }

                street_elements = formatted_address[-3].split(" ")
                if street_elements.count > 1
                    address_hash[:street] = formatted_address[-3].gsub(formatted_address[-3].split(" ").last, "").strip
                    address_hash[:street_number] = formatted_address[-3].split(" ").last
                else
                    address_hash[:street] = formatted_address[-3].strip
                end

                # Check whether the results were plausible. Return nothing if in doubt.
                if address_hash[:country_code] == "DE"
                    # Postal code must be a number
                    Integer(address_hash[:postal_code])
                end
                address_hash
            rescue Exception
                return {}
            end

        end


    end
end
